@@ -88,6 +88,9 @@ QBluetoothDeviceDiscoveryAgentPrivate::~QBluetoothDeviceDiscoveryAgentPrivate()
     if (m_active != NoScanActive)
         stop();
 
+    if (leScanner.isValid())
+        leScanner.setField<jlong>("qtObject", reinterpret_cast<jlong>(nullptr));
+
     if (receiver) {
         receiver->unregisterReceiver();
         delete receiver;
@@ -147,10 +150,8 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start(QBluetoothDeviceDiscoveryAgent
     }
 
     // check Android v23+ permissions
-    // -> BTLE search requires android.permission.ACCESS_COARSE_LOCATION or android.permission.ACCESS_FINE_LOCATION
-    if (requestedMethods & QBluetoothDeviceDiscoveryAgent::LowEnergyMethod
-        && QtAndroid::androidSdkVersion() >= 23)
-    {
+    // -> any device search requires android.permission.ACCESS_COARSE_LOCATION or android.permission.ACCESS_FINE_LOCATION
+    if (QtAndroid::androidSdkVersion() >= 23) {
         const QString coarsePermission(QLatin1String("android.permission.ACCESS_COARSE_LOCATION"));
         const QString finePermission(QLatin1String("android.permission.ACCESS_FINE_LOCATION"));
 
@@ -184,6 +185,44 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start(QBluetoothDeviceDiscoveryAgent
         qCWarning(QT_BT_ANDROID) << "ACCESS_COARSE|FINE_LOCATION permission available";
     }
 
+    // Double check Location service is turned on
+    bool locationTurnedOn = true; // backwards compatible behavior to previous Qt versions
+    const  QAndroidJniObject locString = QAndroidJniObject::getStaticObjectField(
+                "android/content/Context", "LOCATION_SERVICE", "Ljava/lang/String;");
+    const QAndroidJniObject locService = QtAndroid::androidContext().callObjectMethod(
+                "getSystemService",
+                "(Ljava/lang/String;)Ljava/lang/Object;",
+                locString.object<jstring>());
+
+    if (locService.isValid()) {
+        if (QtAndroid::androidSdkVersion() >= 28) {
+            locationTurnedOn = bool(locService.callMethod<jboolean>("isLocationEnabled"));
+        } else {
+            // try GPS and network provider
+            QAndroidJniObject provider = QAndroidJniObject::getStaticObjectField(
+                        "android/location/LocationManager", "GPS_PROVIDER", "Ljava/lang/String;");
+            bool gpsTurnedOn = bool(locService.callMethod<jboolean>("isProviderEnabled",
+                                      "(Ljava/lang/String;)Z", provider.object<jstring>()));
+
+            provider = QAndroidJniObject::getStaticObjectField(
+                       "android/location/LocationManager", "NETWORK_PROVIDER", "Ljava/lang/String;");
+            bool providerTurnedOn = bool(locService.callMethod<jboolean>("isProviderEnabled",
+                                          "(Ljava/lang/String;)Z", provider.object<jstring>()));
+
+            locationTurnedOn = gpsTurnedOn || providerTurnedOn;
+        }
+    }
+
+    if (!locationTurnedOn) {
+        qCWarning(QT_BT_ANDROID) << "Search not possible due to turned off Location service";
+        lastError = QBluetoothDeviceDiscoveryAgent::UnknownError;
+        errorString = QBluetoothDeviceDiscoveryAgent::tr("Location service turned off. Search is not possible.");
+        emit q->error(lastError);
+        return;
+    }
+
+    qCDebug(QT_BT_ANDROID) << "Location turned on";
+
     // install Java BroadcastReceiver
     if (!receiver) {
         // SDP based device discovery
@@ -200,17 +239,25 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start(QBluetoothDeviceDiscoveryAgent
     if (requestedMethods & QBluetoothDeviceDiscoveryAgent::ClassicMethod) {
         const bool success = adapter.callMethod<jboolean>("startDiscovery");
         if (!success) {
-            lastError = QBluetoothDeviceDiscoveryAgent::InputOutputError;
-            errorString = QBluetoothDeviceDiscoveryAgent::tr("Classic Discovery cannot be started");
-            emit q->error(lastError);
+            qCDebug(QT_BT_ANDROID) << "Classic Discovery cannot be started";
+            if (requestedMethods == QBluetoothDeviceDiscoveryAgent::ClassicMethod) {
+                //only classic discovery requested -> error out
+                lastError = QBluetoothDeviceDiscoveryAgent::InputOutputError;
+                errorString = QBluetoothDeviceDiscoveryAgent::tr("Classic Discovery cannot be started");
+
+                emit q->error(lastError);
+                return;
+            } // else fall through to LE discovery
+        } else {
+            m_active = SDPScanActive;
+            qCDebug(QT_BT_ANDROID)
+                << "QBluetoothDeviceDiscoveryAgentPrivate::start() - Classic search successfully started.";
             return;
         }
+    }
 
-        m_active = SDPScanActive;
-        qCDebug(QT_BT_ANDROID)
-            << "QBluetoothDeviceDiscoveryAgentPrivate::start() - Classic search successfully started.";
-    } else {
-        // LE search only requested
+    if (requestedMethods & QBluetoothDeviceDiscoveryAgent::LowEnergyMethod) {
+        // LE search only requested or classic discovery failed but lets try LE scan anyway
         Q_ASSERT(requestedMethods & QBluetoothDeviceDiscoveryAgent::LowEnergyMethod);
 
         if (QtAndroidPrivate::androidSdkVersion() < 18) {
@@ -317,25 +364,51 @@ void QBluetoothDeviceDiscoveryAgentPrivate::processDiscoveredDevices(
 
     for (int i = 0; i < discoveredDevices.size(); i++) {
         if (discoveredDevices[i].address() == info.address()) {
-            if (discoveredDevices[i] == info && lowEnergySearchTimeout > 0) {
-                qCDebug(QT_BT_ANDROID) << "Duplicate: " << info.address()
-                                       << "isLeScanResult:" << isLeResult;
+            QBluetoothDeviceInfo::Fields updatedFields = QBluetoothDeviceInfo::Field::None;
+            if (discoveredDevices[i].rssi() != info.rssi()) {
+                qCDebug(QT_BT_ANDROID) << "Updating RSSI for" << info.address()
+                                       << info.rssi();
+                discoveredDevices[i].setRssi(info.rssi());
+                updatedFields.setFlag(QBluetoothDeviceInfo::Field::RSSI);
+            }
+            if (discoveredDevices[i].manufacturerData() != info.manufacturerData()) {
+                qCDebug(QT_BT_ANDROID) << "Updating manufacturer data for" << info.address();
+                const QVector<quint16> keys = info.manufacturerIds();
+                for (auto key: keys)
+                    discoveredDevices[i].setManufacturerData(key, info.manufacturerData(key));
+                updatedFields.setFlag(QBluetoothDeviceInfo::Field::ManufacturerData);
+            }
+
+            if (lowEnergySearchTimeout > 0) {
+                if (discoveredDevices[i] != info) {
+                    if (discoveredDevices.at(i).name() == info.name()) {
+                        qCDebug(QT_BT_ANDROID) << "Almost Duplicate " << info.address()
+                                               << info.name() << "- replacing in place";
+                        discoveredDevices.replace(i, info);
+                        emit q->deviceDiscovered(info);
+                    }
+                } else {
+                    if (!updatedFields.testFlag(QBluetoothDeviceInfo::Field::None))
+                        emit q->deviceUpdated(discoveredDevices[i], updatedFields);
+                }
+
                 return;
             }
 
-            if (discoveredDevices.at(i).name() == info.name()) {
-                qCDebug(QT_BT_ANDROID) << "Almost Duplicate "<< info.address()
-                                       << info.name() << "- replacing in place";
-                discoveredDevices.replace(i, info);
-                emit q->deviceDiscovered(info);
-                return;
-            }
+            discoveredDevices.replace(i, info);
+            emit q->deviceDiscovered(info);
+
+            if (!updatedFields.testFlag(QBluetoothDeviceInfo::Field::None))
+                emit q->deviceUpdated(discoveredDevices[i], updatedFields);
+
+            return;
         }
     }
 
     discoveredDevices.append(info);
     qCDebug(QT_BT_ANDROID) << "Device found: " << info.name() << info.address().toString()
-                           << "isLeScanResult:" << isLeResult;
+                           << "isLeScanResult:" << isLeResult
+                           << "Manufacturer data size:" << info.manufacturerData().size();
     emit q->deviceDiscovered(info);
 }
 
